@@ -37,9 +37,14 @@ class TestBuilderMethods(unittest.TestCase):
 
 class BaseBuilderNodesTest(object):
 
+    def setUp(self):
+        if not self.adapter.is_available():
+            self.skipTest('Library for adapter %s is not available'
+            % self.adapter)
+
     @property
     def my_builder(self):
-        raise NotImplementedError()
+        return functools.partial(xml4h.build, adapter=self.adapter)
 
     def test_element(self):
         xmlb = self.my_builder('DocRoot')
@@ -318,31 +323,29 @@ class BaseBuilderNodesTest(object):
             '    <NSDefaultExplicit xmlns="urn:default"/>\n'
             '    <%sNSCustomExplicit xmlns="urn:custom"/>\n'
             '    <myns:NSCustomWithPrefixImplicit/>\n'
-            '    <myns:NSCustomWithPrefixExplicit xmlns="urn:custom"/>\n'
+            '    <%sNSCustomWithPrefixExplicit xmlns="urn:custom"/>\n'
             '    <Attrs1 default-ns-explicit="1"'
                        ' default-ns-implicit="1"/>\n'
             '    <Attrs2'
                        ' myns:custom-ns-prefix-explicit="1"'
                        ' myns:custom-ns-prefix-implicit="1"/>\n'
             '</DocRoot>\n'
-            % (self.adapter == xml4h.LXMLAdapter and 'myns:' or ''))
-            # TODO: lxml outputs prefix in more situations than minidom
+            % (self.adapter == xml4h.LXMLAdapter and 'myns:' or '',
+               self.adapter not in (
+                   xml4h.ElementTreeAdapter, xml4h.cElementTreeAdapter)
+                   and 'myns:' or ''))
+            # TODO: Any way to make lxml/ElementTree output more consistent?
         self.assertEqual(xml, xmlb.dom_element.xml_doc())
         # Test namespaces work as expected when searching/traversing DOM
         self.assertEqual(
             ['DocRoot', 'NSDefaultImplicit', 'NSDefaultExplicit',
              'Attrs1', 'Attrs2'],
             [n.name for n in xmlb.find_doc(ns_uri='urn:default')])
-        if self.adapter == xml4h.LXMLAdapter:  # TODO: Differing prefix output
-            self.assertEqual(
-                ['myns:NSCustomExplicit', 'myns:NSCustomWithPrefixImplicit',
-                 'myns:NSCustomWithPrefixExplicit'],
-                [n.name for n in xmlb.find_doc(ns_uri='urn:custom')])
-        else:
-            self.assertEqual(
-                ['NSCustomExplicit', 'myns:NSCustomWithPrefixImplicit',
-                 'myns:NSCustomWithPrefixExplicit'],
-                [n.name for n in xmlb.find_doc(ns_uri='urn:custom')])
+        self.assertEqual(
+            ['NSCustomExplicit',
+             'NSCustomWithPrefixImplicit',
+             'NSCustomWithPrefixExplicit'],
+            [n.local_name for n in xmlb.find_doc(ns_uri='urn:custom')])
         self.assertEqual(
             ['NSCustomExplicit', 'NSCustomWithPrefixImplicit',
              'NSCustomWithPrefixExplicit'],
@@ -399,10 +402,10 @@ class BaseBuilderNodesTest(object):
             xmlb.dom_element.xml_doc())
         # Attempts to use undefined namespace prefixes will fail
         xmlb = self.my_builder('DocRoot', ns_uri='urn:default')
-        self.assertRaises(Exception,
-            xmlb.e, ['missingns:Elem1'])
-        self.assertRaises(Exception,
-            xmlb.attrs, [{'missingns:attrib1': 'value1'}])
+        self.assertRaises(xml4h.exceptions.UnknownNamespaceException,
+            xmlb.e, 'missingns:Elem1')
+        self.assertRaises(xml4h.exceptions.UnknownNamespaceException,
+            xmlb.attrs, {'missingns:attrib1': 'value1'})
         # Element with literal namespace defn will use the ns as its default
         xmlb = self.my_builder('DocRoot', ns_uri='urn:default')
         xmlb.e('{urn:missing}Elem1')
@@ -447,8 +450,9 @@ class BaseBuilderNodesTest(object):
                 .e('Elem1').t('<content/> as text').up()
                 .e('Elem2').d('<content/> as cdata').up()
             )
-        if self.adapter == xml4h.LXMLAdapter:
-            # TODO: lxml library does not support real cdata?
+        if self.adapter in (xml4h.LXMLAdapter, xml4h.ElementTreeAdapter,
+                            xml4h.cElementTreeAdapter):
+            # TODO: Make lxml & ElementTree libs support real cdata
             self.assertEqual(
                 '<?xml version="1.0" encoding="utf-8"?>\n'
                 '<DocRoot>\n'
@@ -542,17 +546,17 @@ class BaseBuilderNodesTest(object):
         to another using methods on the xml4h Node.
         """
         cat_b = (
-            xml4h.build('Animal')
+            self.my_builder('Animal')
                 .element('Cat')
                     .element('Feature').text('Independent')
             )
         dog_b = (
-            xml4h.build('Animal')
+            self.my_builder('Animal')
                 .element('Dog')
                     .element('Feature').text('Loyal')
             )
         horse_b = (
-            xml4h.build('Animal')
+            self.my_builder('Animal')
                 .element('Horse')
                     .element('Feature').text('Transport')
             )
@@ -592,17 +596,17 @@ class BaseBuilderNodesTest(object):
         document to another using builder methods.
         """
         cat_b = (
-            xml4h.build('Animal')
+            self.my_builder('Animal')
                 .element('Cat')
                     .element('Feature').text('Independent')
             )
         dog_b = (
-            xml4h.build('Animal')
+            self.my_builder('Animal')
                 .element('Dog')
                     .element('Feature').text('Loyal')
             )
         horse_b = (
-            xml4h.build('Animal')
+            self.my_builder('Animal')
                 .element('Horse')
                     .element('Feature').text('Transport')
             )
@@ -642,7 +646,7 @@ class BaseBuilderNodesTest(object):
         documentation.
         """
         # Create builder with the name of the root element
-        b = (xml4h.build('MontyPythonFilms')
+        b = (self.my_builder('MontyPythonFilms')
             # Assign attributes to the new root element
             .attributes(
                 {'source': 'http://en.wikipedia.org/wiki/Monty_Python'})
@@ -743,10 +747,6 @@ class TestXmlDomBuilder(BaseBuilderNodesTest, unittest.TestCase):
     def adapter(self):
         return xml4h.XmlDomImplAdapter
 
-    @property
-    def my_builder(self):
-        return functools.partial(xml4h.build, adapter=self.adapter)
-
 
 class TestLXMLEtreeBuilder(BaseBuilderNodesTest, unittest.TestCase):
     """
@@ -755,10 +755,24 @@ class TestLXMLEtreeBuilder(BaseBuilderNodesTest, unittest.TestCase):
 
     @property
     def adapter(self):
-        if not xml4h.LXMLAdapter.is_available():
-            self.skipTest("lxml library is not installed")
         return xml4h.LXMLAdapter
 
+
+class TestElementTreeBuilder(BaseBuilderNodesTest, unittest.TestCase):
+    """
+    Test building with the (c)ElementTree library.
+    """
+
     @property
-    def my_builder(self):
-        return functools.partial(xml4h.build, adapter=self.adapter)
+    def adapter(self):
+        return xml4h.ElementTreeAdapter
+
+
+class TestcElementTreeBuilder(BaseBuilderNodesTest, unittest.TestCase):
+    """
+    Test building with the (c)ElementTree library.
+    """
+
+    @property
+    def adapter(self):
+        return xml4h.cElementTreeAdapter
