@@ -10,6 +10,13 @@ class Xml4hException(Exception):
     pass
 
 
+class Xml4hImplementationBug(Xml4hException):
+    """
+    *xml4h* implementation has a bug, probably.
+    """
+    pass
+
+
 class FeatureUnavailableException(Xml4hException):
     """
     User has attempted to use a feature that is available in some *xml4h*
@@ -28,3 +35,11 @@ class IncorrectArgumentTypeException(ValueError, Xml4hException):
         msg = (u'Argument %s is not one of the expected types: %s'
             % (arg, expected_types))
         super(IncorrectArgumentTypeException, self).__init__(msg)
+
+
+class UnknownNamespaceException(ValueError, Xml4hException):
+    """
+    User has attempted to refer to an unknown or undeclared namespace by
+    prefix or URI.
+    """
+    pass
