@@ -2,7 +2,7 @@ import re
 import copy
 
 from xml4h.impls.interface import XmlImplAdapter
-from xml4h import nodes
+from xml4h import nodes, exceptions
 
 try:
     from lxml import etree
@@ -72,7 +72,7 @@ class LXMLAdapter(XmlImplAdapter):
                 return nodes.CDATA
             else:
                 return nodes.Text
-        raise Exception(
+        raise exceptions.Xml4hImplementationBug(
             'Unrecognized type for implementation node: %s' % node)
 
     def get_impl_root(self, node):
@@ -141,11 +141,12 @@ class LXMLAdapter(XmlImplAdapter):
         converted to the prefix name '_' so it can be used despite empty
         namespace prefixes being unsupported by XPath.
         """
-        try:
+        if isinstance(node, etree._ElementTree):
+            # Document node lxml.etree._ElementTree has no nsmap, lookup root
+            root = self.get_impl_root(node)
+            namespaces_dict = root.nsmap.copy()
+        else:
             namespaces_dict = node.nsmap.copy()
-        except AttributeError:
-            # Document node lxml.etree._ElementTree has no nsmap
-            namespaces_dict = {}
         if 'namespaces' in kwargs:
             namespaces_dict.update(kwargs['namespaces'])
         # Empty namespace prefix is not supported, convert to '_' prefix
@@ -207,9 +208,6 @@ class LXMLAdapter(XmlImplAdapter):
         else:
             return self.get_node_local_name(node)
 
-    def set_node_name(self, node, name):
-        raise NotImplementedError()
-
     def get_node_local_name(self, node):
         return re.sub('{.*}', '', node.tag)
 
@@ -304,8 +302,9 @@ class LXMLAdapter(XmlImplAdapter):
         elif ':' in name:
             prefix, name = name.split(':')
             if prefix == 'xmlns':
-                prefix = None
-            name = '{%s}%s' % (element.nsmap[prefix], name)
+                name = '{%s}%s' % (nodes.Node.XMLNS_URI, name)
+            else:
+                name = '{%s}%s' % (element.nsmap[prefix], name)
         if name in element.attrib:
             del(element.attrib[name])
 
@@ -329,7 +328,7 @@ class LXMLAdapter(XmlImplAdapter):
                 parent.append(child)
             return child
 
-    def import_node(self, parent, node, clone=False):
+    def import_node(self, parent, node, original_parent=None, clone=False):
         original_node = node
         if clone:
             node = self.clone_node(node)
@@ -401,7 +400,7 @@ class LXMLAdapter(XmlImplAdapter):
                 for n, v in curr_node.attrib.items():
                     if v == uri and ('{%s}' % nodes.Node.XMLNS_URI) in n:
                         result = n.split('}')[1]
-                        break
+                        return result
                 curr_node = self.get_node_parent(curr_node)
         return result
 
