@@ -37,8 +37,12 @@ class Node(object):
             subclass implementation to mediate operations on the node in
             the underlying XML implementation.
         """
-        if node is None or adapter is None:
-            raise Exception('Cannot instantiate without node and adapter')
+        if node is None:
+            raise xml4h.exceptions.IncorrectArgumentTypeException(
+                node, [object])
+        if adapter is None:
+            raise xml4h.exceptions.IncorrectArgumentTypeException(
+                adapter, [object])
         self._impl_node = node
         self._adapter = adapter
 
@@ -56,7 +60,8 @@ class Node(object):
 
     def __str__(self):
         # TODO Degrade non-ASCII characters gracefully
-        return str(self.__unicode__())
+        # Avoid keyword args in encode call for compatibility with python 2.6
+        return self.__unicode__().encode('ascii', 'replace')
 
     def __repr__(self):
         return self.__str__()
@@ -121,7 +126,8 @@ class Node(object):
         if self.is_root:
             return self
         return self.adapter.wrap_node(
-            self.adapter.impl_root_element, self.adapter.impl_document)
+            self.adapter.impl_root_element, self.adapter.impl_document,
+            self.adapter)
 
     @property
     def is_root(self):
@@ -231,8 +237,9 @@ class Node(object):
         Convert a list of underlying implementation nodes into a list of
         *xml4h* wrapper nodes.
         """
-        nodelist = [self.adapter.wrap_node(n, self.adapter.impl_document)
-                    for n in impl_nodelist]
+        nodelist = [
+            self.adapter.wrap_node(n, self.adapter.impl_document, self.adapter)
+            for n in impl_nodelist]
         return NodeList(nodelist)
 
     @property
@@ -242,7 +249,7 @@ class Node(object):
         """
         parent_impl_node = self.adapter.get_node_parent(self.impl_node)
         return self.adapter.wrap_node(
-            parent_impl_node, self.adapter.impl_document)
+            parent_impl_node, self.adapter.impl_document, self.adapter)
 
     @property
     def ancestors(self):
@@ -347,7 +354,7 @@ class Node(object):
             self.adapter.get_node_parent(self.impl_node), self.impl_node,
             destroy_node=destroy)
         if removed_child is not None:
-            return self.adapter.wrap_node(removed_child, None)
+            return self.adapter.wrap_node(removed_child, None, self.adapter)
         else:
             return None
 
@@ -368,7 +375,7 @@ class Node(object):
             child_impl_node = node  # Assume it's a valid impl node
         self.adapter.import_node(self.impl_node, child_impl_node, clone=True)
 
-    def transplant_node(self, node, copy=False):
+    def transplant_node(self, node):
         """
         Transplant a node from another document to become a child of this node,
         removing it from the source document.  The node to be transplanted can
@@ -380,9 +387,12 @@ class Node(object):
         """
         if isinstance(node, xml4h.nodes.Node):
             child_impl_node = node.impl_node
+            original_parent_impl_node = node.parent.impl_node
         else:
             child_impl_node = node  # Assume it's a valid impl node
-        self.adapter.import_node(self.impl_node, child_impl_node, clone=False)
+            original_parent_impl_node = self.adapter.get_node_parent(node)
+        self.adapter.import_node(self.impl_node, child_impl_node,
+            original_parent_impl_node, clone=False)
 
     def find(self, name=None, ns_uri=None, first_only=False):
         """
@@ -410,7 +420,7 @@ class Node(object):
         if first_only:
             if impl_nodelist:
                 return self.adapter.wrap_node(
-                    impl_nodelist[0], self.adapter.impl_document)
+                    impl_nodelist[0], self.adapter.impl_document, self.adapter)
             else:
                 return None
         return self._convert_nodelist(impl_nodelist)
@@ -536,17 +546,14 @@ class NodeAttrAndChildElementLookupsMixin(object):
         """
         Retrieve this node's child element by tag name regardless of the
         elements namespace, assuming the name given doesn't match an existing
-        attribute of this class.
+        attribute or method.
 
-        :param string child_name: tag name of the child element. The name must
-            match the following pattern rules for *xml4h* to attempt a child
-            element lookup, otherwise an AttributeError will be raised
-            immediately:
+        :param string child_name: tag name of the child element to look up.
+            To avoid name clashes with class attributes the child name may
+            includes a trailing underscore (``_``) character, which is removed
+            to get the real child tag name.
+            The child name must not begin with underscore characters.
 
-            - name contains one or more uppercase characters, or
-            - name is all lowercase but ends with a single underscore character
-            - **in all cases** the name does not begin with an underscore
-              character.
         :return: the type of the return value depends on how many child
             elements match the name:
 
@@ -559,10 +566,10 @@ class NodeAttrAndChildElementLookupsMixin(object):
         if child_name.startswith('_'):
             # Never match names with underscore leaders, for safety
             pass
-        elif child_name != child_name.lower() or child_name.endswith('_'):
+        else:
+            # If name is munged with trailing underscore, remove it
             if child_name.endswith('_'):
                 child_name = child_name[:-1]
-            # Names with uppercase characters or trailing '_' are fair game
             results = self.children(local_name=child_name, node_type=Element)
             if len(results) == 1:
                 return results[0]
@@ -582,7 +589,8 @@ class XPathMixin(object):
         if isinstance(node, (basestring, int, long, float)):
             return node
         else:
-            return self.adapter.wrap_node(node, self.adapter.impl_document)
+            return self.adapter.wrap_node(
+                node, self.adapter.impl_document, self.adapter)
 
     def xpath(self, xpath, **kwargs):
         """
@@ -790,7 +798,8 @@ class Element(NameValueNodeMixin,
                 for n, v in attr_obj:
                     attr_dict[n] = v
             else:
-                raise Exception('Attribute data must be a dictionary or list')
+                raise xml4h.exceptions.IncorrectArgumentTypeException(
+                    attr_obj, [dict, list, tuple])
 
         # Always process 'xmlns' namespace definitions first, in case other
         # attributes belong to a newly-defined namespace
@@ -810,7 +819,7 @@ class Element(NameValueNodeMixin,
             prefix, name, my_ns_uri = self.adapter.get_ns_info_from_node_name(
                 attr_name, element)
             if ' ' in name:
-                raise Exception("Invalid attribute name value contains space")
+                raise ValueError("Invalid attribute name value contains space")
             # If necessary, add an xmlns defn for new prefix-defined namespace
             if not prefix and '}' in attr_name:
                 prefix = self.adapter.get_ns_prefix_for_uri(
@@ -870,14 +879,14 @@ class Element(NameValueNodeMixin,
         # Add new attributes
         self._set_element_attributes(self.impl_node, attr_obj=attr_obj)
 
-    attrib = set_attributes  # Alias
+    attrib = attributes  # Alias
     """
-    Alias of :meth:`set_attributes`
+    Alias of :meth:`attributes`
     """
 
-    attrs = set_attributes  # Alias
+    attrs = attributes  # Alias
     """
-    Alias of :meth:`set_attributes`
+    Alias of :meth:`attributes`
     """
 
     @property
@@ -888,7 +897,7 @@ class Element(NameValueNodeMixin,
         """
         impl_attr_nodes = self.adapter.get_node_attributes(self.impl_node)
         wrapped_attr_nodes = [
-            self.adapter.wrap_node(a, self.adapter.impl_document)
+            self.adapter.wrap_node(a, self.adapter.impl_document, self.adapter)
             for a in impl_attr_nodes]
         return sorted(wrapped_attr_nodes, key=lambda x: x.name)
 
@@ -904,7 +913,7 @@ class Element(NameValueNodeMixin,
         attr_impl_node = self.adapter.get_node_attribute_node(
             self.impl_node, name, ns_uri)
         return self.adapter.wrap_node(
-            attr_impl_node, self.adapter.impl_document)
+            attr_impl_node, self.adapter.impl_document, self.adapter)
 
     def _add_ns_prefix_attr(self, element, prefix, ns_uri):
         if prefix is None:
@@ -997,7 +1006,8 @@ class Element(NameValueNodeMixin,
         # ...or in the default position, appended after existing nodes
         else:
             self.adapter.add_node_child(self.impl_node, child_elem)
-        return self.adapter.wrap_node(child_elem, self.adapter.impl_document)
+        return self.adapter.wrap_node(
+            child_elem, self.adapter.impl_document, self.adapter)
 
     def _add_text(self, element, text):
         text_node = self.adapter.new_impl_text(text)
@@ -1106,7 +1116,8 @@ class AttributeDict(object):
 
     def __str__(self):
         # TODO Degrade non-ASCII characters gracefully
-        return str(self.__unicode__())
+        # Avoid keyword args in encode call for compatibility with python 2.6
+        return self.__unicode__().encode('ascii', 'replace')
 
     def __repr__(self):
         return self.__str__()
@@ -1168,7 +1179,7 @@ class AttributeDict(object):
         :return: the :class:`Element` that contains these attributes.
         """
         return self.adapter.wrap_node(
-            self.impl_element, self.adapter.impl_document)
+            self.impl_element, self.adapter.impl_document, self.adapter)
 
     @property
     def impl_attributes(self):
